import { useState } from "react";
import { apiInstance } from "@/shared/config/apiConfig";
import { useSession } from "next-auth/react";
import { AxiosError } from "axios";
import Loader from "@/shared/ui/Loader/Loader";
import ChangePasswordInput from "./ChangePasswordInput/ChangePasswordInput";
import { Link } from "@/shared/config/navigation";
import { z } from "zod";
import { zodResolver } from "@hookform/resolvers/zod";
import { useForm } from "react-hook-form";

interface IChangePasswordProps {
  closeWindow: (bool: boolean) => void;
}

const ChangePassword: React.FC<IChangePasswordProps> = ({
  closeWindow,
}: IChangePasswordProps) => {
  const session = useSession();
  const [oldPassword, setOldPassword] = useState<string>("");
  const [newPassword, setNewPassword] = useState<string>("");
  const [confirmNewPassword, setConfirmNewPassword] = useState<string>("");

  const [warningOldPassword, setWarningOldPassword] = useState<string>("");
  const [warningNewPassword, setWarningNewPassword] = useState<string>("");
  const [warningConfirmNewPassword, setWarningConfirmNewPassword] =
    useState<string>("");
  const [error, setError] = useState<string>("");
  const [loader, setLoader] = useState<boolean>(false);
  const [success, setSuccess] = useState<boolean>(false);

  const changePasswordScheme = z
    .object({
      old_password: z.string().min(8, "Минимум 8 символов"),
      new_password1: z.string().min(8, "Минимум 8 символов"),
      new_password2: z.string().min(8, "Минимум 8 символов"),
    })
    .refine((data) => data.new_password1 === data.new_password2, {
      message: "Пароли не совпадают",
      path: ["new_password2"],
    });
  type FormFields = z.infer<typeof changePasswordScheme>;

  const {
    register,
    handleSubmit,
    formState: { errors },
  } = useForm<FormFields>({
    resolver: zodResolver(changePasswordScheme),
  });

  const onSubmit = async (data: FormFields) => {
    try {
      setError("");
      setLoader(true);

      const Authorization = `Bearer ${session.data?.access_token}`;
      const config = {
        headers: {
          Authorization,
        },
      };
      const res = await apiInstance.patch(
        "/auth/password_change/",
        data,
        config
      );

      if ([200, 201].includes(res.status)) return setSuccess(true);
    } catch (error: unknown) {
      if (error instanceof AxiosError) {
        if (error.response?.status === 400) {
          setError("Некорректный старый пароль или недопустимый новый пароль");
        }
      } else {
        setError("Произошла непредвиденная ошибка");
      }
    } finally {
      setLoader(false);
    }
  };

  return (
    <div
      onClick={() => closeWindow(false)}
      className="w-full h-full fixed top-0 left-0 z-10 bg-gray-950 flex items-center justify-center"
    >
      <div
        onClick={(e) => e.stopPropagation()}
        className="flex w-full max-w-[400px] p-6 flex-col gap-4 rounded-md bg-white"
      >
        <form onSubmit={handleSubmit(onSubmit)}>
          <h4 className="mb-2 text-[18px] font-bold leading-7 text-gray-900">
            Изменить пароль
          </h4>
          <ChangePasswordInput
            onChange={(e) => setOldPassword(e.target.value)}
            value={oldPassword}
            placeholder="Введите старый пароль"
            label="Старый пароль"
            error={warningOldPassword}
          />
          <Link
            href="/sign-in/forgot-password"
            className="self-end leading-8 text-blue"
          >
            Забыли пароль?
          </Link>
          <ChangePasswordInput
            onChange={(e) => setNewPassword(e.target.value)}
            value={newPassword}
            placeholder="Введите новый пароль"
            label="Новый пароль"
            error={warningNewPassword}
          />
          <ChangePasswordInput
            onChange={(e) => setConfirmNewPassword(e.target.value)}
            value={confirmNewPassword}
            placeholder="Повторите новый пароль"
            label="Потвердить новый пароль"
            error={warningConfirmNewPassword}
          />
          {error ? <p className="text-red-500">{error}</p> : null}
          {success ? (
            <p className="text-light-blue">Вы успешно поменяли пароль!</p>
          ) : null}

          <div className="mt-9 flex flex-col gap-3">
            <button
              type="button"
              className="py-3 px-4 rounded-md font-bold leading-6 shadow-sm border border-blue bg-blue text-white"
            >
              {loader ? <Loader /> : "Сохранить"}
            </button>
            <button
              className="py-3 px-4 rounded-md font-bold leading-6 shadow-md border border-gray-300 bg-slate-200 text-gray-500"
              type="button"
              onClick={() => closeWindow(false)}
            >
              Отмена
            </button>
          </div>
        </form>
      </div>
    </div>
  );
};

export default ChangePassword;
